@@ -59,10 +59,10 @@ const char *EXTERNALS_PATHS[] = {"~/.extsmail/externals",
 #define LOCKFILE "extsmaild.lock"
 
 // Size of the initial header buffer (and also the size that it will be
-// incremented if it's not big enough).
+// incremented by if it's not big enough).
 #define HEADER_BUF 2048
 // Size of the initial stderr buffer (and also the size that it will be
-// incremented if it's not big enough).
+// incremented by if it's not big enough).
 #define STDERR_BUF_ALLOC 1024
 // Size of the initial buffer copying data from a message to the child
 // process.
@@ -107,29 +107,21 @@ void obtain_lock(Conf *conf)
     // a lock on it then we assume the lock file is stale and that there's no
     // running extsmaild.
 
-    if ((lock_fd = open(lock_path, O_CREAT | O_RDWR)) == -1) {
-        if (errno != EEXIST)
-            err(1, "Unable to obtain lockfile '%s'", lock_path);
+    if (((lock_fd = open(lock_path, O_CREAT | O_RDWR, 0600)) == -1)
+     && errno != EEXIST) {
+        err(1, "Unable to obtain lockfile '%s'", lock_path);
     }
 
     if (flock(lock_fd, LOCK_EX | LOCK_NB) == -1) {
         err(1, "Unable to obtain lockfile '%s'", lock_path);
     }
 
-    fchmod(lock_fd, S_IRUSR | S_IWUSR | S_IXUSR);
-    
     // Install an exit handler and signal trap so that, barring something going
     // catastrophically wrong, the lockfile is removed on exit.
     
     atexit(lock_exit);
     signal(SIGINT, sigterm_trap);
     signal(SIGTERM, sigterm_trap);
-    
-    // In our context, SIGPIPE would occur when a write to a pipe fails, causing
-    // us to terminate. Therefore we ignore SIGPIPE's, which means that calls to
-    // write will return -1 if EPIPE occurs.
-    
-    signal(SIGPIPE, SIG_IGN);
 }
 
 
@@ -312,10 +304,10 @@ bool cycle(Conf *conf, Group *groups)
             }
 
             if (flock(fd, LOCK_EX | LOCK_NB) == -1) {
-                if (errno == EWOULDBLOCK)
-                    goto next;
+                if (errno != EWOULDBLOCK)
+                    syslog(LOG_ERR, "Error when flocking'ing '%s'", msg_path);
 
-                err(1, "cycle: flock: when locking spool file %s", msg_path);
+                goto next;
             }
 
             // We've now got an exclusive lock on the file.
@@ -323,12 +315,14 @@ bool cycle(Conf *conf, Group *groups)
             if (fstat(fd, &msg_st) == -1) {
                 // If stat failed then something odd has happened and it's
                 // best to bail out for the time being.
+
+                syslog(LOG_ERR, "Error when fstat'ing '%s'", msg_path);
                 close(fd);
                 all_sent = false;
                 break;
             }
 
-            // If the file is zero size which means that we managed to
+            // If the file is zero size it probably means that we managed to
             // interrupt extsmail before it had managed to obtain an
             // exclusive lock and write any data to the file. In such a case
             // we don't try and do anything; instead (below) we sleep a
@@ -348,9 +342,9 @@ bool cycle(Conf *conf, Group *groups)
                 }
             }
 
-            close(fd);
-            
 next:
+            close(fd);
+
             // At this point, either we've released the exclusive lock we'd
             // previously gained (because we'd gained it before any data had
             // been written to the spool file) or we weren't able to gain the
@@ -451,7 +445,7 @@ bool try_groups(Conf *conf, Group *groups, const char *msg_path, int fd)
     off_t mf_body_off = lseek(fd, 0, SEEK_CUR);
     if (mf_body_off == -1) {
         syslog(LOG_ERR, "Error when ftell'ing from '%s'", msg_path);
-        goto fail;
+        goto preheaderfail;
     }
 
     // Read in the messages header, doctoring it along the way to make it
@@ -567,8 +561,8 @@ next_group:
         goto fail;
     }
 
-    // OK, at this point we've found a matching group for the message. We now
-    // go through each external in the group and try
+    // At this point we've found a matching group for the message. We now go
+    // through each external in the group and try to use it to send the message.
 
     External *cur_ext = group->externals;
     while (cur_ext != NULL) {
@@ -576,8 +570,9 @@ next_group:
         // but which hasn't previously been tried, then we set its 'last_success'
         // value to the current time so that timeouts are measured from the first
         // time it is used.
-        if (conf->mode == DAEMON_MODE && cur_ext->timeout != 0 &&
-          cur_ext->last_success == 0) {
+        if (conf->mode == DAEMON_MODE
+          && cur_ext->timeout != 0
+          && cur_ext->last_success == 0) {
             assert(cur_ext->working);
             cur_ext->last_success = time(NULL);
         }
@@ -599,18 +594,18 @@ next_group:
             }
 
             // Don't bother trying to use this external. This is an important
-            // optimisation: if a host is down, it can take quite a while for
-            // network connections to time out, so we don't want to continually
-            // retry something that already hasn't worked. On the next cycle,
-            // the "working" flag will be reset.
+            // optimisation if using ssh for example: if a host is down, it can
+            // take quite a while for network connections to time out, so we
+            // don't want to continually retry something that already hasn't
+            // worked. On the next cycle, the "working" flag will be reset.
 
             cur_ext = cur_ext->next;
             continue;
         }
 
         // OK, we're now ready to invoke the sendmail command for this
-        // external. In order to do this, we fork, with the child process
-        // executing the sendmail command. We pipe the message into the
+        // external. In order to do this, we first fork, with the child process
+        // executing the sendmail command. We then pipe the message into the
         // sendmail process, and pipe stderr out from the child (so that we can
         // report errors to the user).
 
@@ -632,7 +627,7 @@ next_group:
             if (dup2(pipeto[0], STDIN_FILENO) == -1 || dup2(pipefrom[1],
               STDERR_FILENO) == -1) {
                 syslog(LOG_CRIT, "try_groups: dup2: %m");
-                exit(1);
+                goto fail;
             }
             close(pipeto[0]);
             close(pipeto[1]);
@@ -654,7 +649,7 @@ next_group:
 
             execvp(cur_ext->sendmail_argv[0], (char **const) sub_argv);
             syslog(LOG_CRIT, "try_groups: execvp: %m");
-            exit(1);
+            goto fail;
         }
         else {
             // Parent process.
@@ -684,7 +679,7 @@ next_group:
                     ssize_t nr = read(fd, buf, PTOC_BUFLEN);
                     if (nr == 0) {
                         // It might look like we'd like to close(fd) now, but
-                        // it's still possible for things to go wrong and fd
+                        // it's still possible for things to go wrong, with fd
                         // needing to be tried again.
                         eof_fd = true;
                         close(pipeto[1]);
@@ -736,7 +731,9 @@ next_group:
             // For reasons that I don't pretend to understand, stderr messages
             // sometimes have random newline chars at the end of line - this loop
             // chomps them off.
-            while (stderr_buf_len > 0 && (stderr_buf[stderr_buf_len - 1] == '\n'
+
+            while (stderr_buf_len > 0
+              && (stderr_buf[stderr_buf_len - 1] == '\n'
               || stderr_buf[stderr_buf_len - 1] == '\r'))
                 stderr_buf_len -= 1;
 
@@ -795,8 +792,8 @@ next:
                 // timeout hasn't been exceeded, then we have to give up on
                 // trying to send this messages via this, or other, externals -
                 // we need to wait for the timeout to be exceeded.       
-                if (cur_ext->timeout != 0 &&
-                  cur_ext->last_success + cur_ext->timeout > time(NULL)) {
+                if (cur_ext->timeout != 0
+                  && cur_ext->last_success + cur_ext->timeout > time(NULL)) {
                     goto fail;
                 }
             }
@@ -806,11 +803,12 @@ next:
     }
 
 fail:
+    free(dhd_buf);
+preheaderfail:
     for (int j = 0; j < nargv; j += 1)
         free(argv[j]);
     free(argv);
     free(stderr_buf);
-    free(dhd_buf);
 
     return false;
 }
@@ -857,6 +855,13 @@ int main(int argc, char** argv)
     
     obtain_lock(conf);
 
+    // In our context, SIGPIPE would occur when a write to a pipe fails, causing
+    // us to terminate. Therefore we ignore SIGPIPE's, which means that calls to
+    // write will return -1 if EPIPE occurs. Why this isn't the default behaviour
+    // is anyone's guess.
+    
+    signal(SIGPIPE, SIG_IGN);
+
     // Check that everything to do with the spool dir is OK.
 
     if (!check_spool_dir(conf))
@@ -898,7 +903,7 @@ int main(int argc, char** argv)
         struct kevent events;
         EV_SET(&changes, smf, EVFILT_VNODE,
           EV_ADD | EV_ENABLE | EV_ONESHOT,
-          NOTE_DELETE | NOTE_WRITE | NOTE_EXTEND | NOTE_TRUNCATE | NOTE_ATTRIB
+          NOTE_DELETE | NOTE_WRITE | NOTE_EXTEND | NOTE_ATTRIB
           | NOTE_LINK | NOTE_RENAME | NOTE_REVOKE,
           0, 0);
 #elif HAVE_INOTIFY
